C-----------------------------------------------------------------------
      FUNCTION DISTANCE(RLAT1,RLON1,RLAT2,RLON2)
C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C
C SUBPROGRAM:  GCDIST     COMPUTE GREAT CIRCLE DISTANCE
C   PRGMMR: IREDELL       ORG: W/NMC23       DATE: 96-04-10
C
C ABSTRACT: THIS SUBPROGRAM COMPUTES GREAT CIRCLE DISTANCE
C           BETWEEN TWO POINTS ON THE EARTH.
C
C PROGRAM HISTORY LOG:
C   96-04-10  IREDELL
C
C USAGE:    ...GCDIST(RLAT1,RLON1,RLAT2,RLON2)
C
C   INPUT ARGUMENT LIST:
C     RLAT1    - REAL LATITUDE OF POINT 1 IN DEGREES
C     RLON1    - REAL LONGITUDE OF POINT 1 IN DEGREES
C     RLAT2    - REAL LATITUDE OF POINT 2 IN DEGREES
C     RLON2    - REAL LONGITUDE OF POINT 2 IN DEGREES
C
C   OUTPUT ARGUMENT LIST:
C     DISTANCE - REAL GREAT CIRCLE DISTANCE IN KILOMETERS
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C
C$$$
      REAL RLAT1,RLON1,RLAT2,RLON2
      DOUBLE PRECISION RERTH,PI,DPR,CLAT1,CLAT2,SLAT2,CDLON,CRD
      PARAMETER(RERTH=6.3712E6)
      PARAMETER(PI=3.14159265358979,DPR=180./PI)
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      IF ((ABS(RLAT1-RLAT2).LT.0.03).AND.
     +    (ABS(RLON1-RLON2).LT.0.03)) THEN
        DISTANCE=0.
      ELSE
        CLAT1=DCOS(RLAT1/DPR)
        SLAT1=DSIN(RLAT1/DPR)
        CLAT2=DCOS(RLAT2/DPR)
        SLAT2=DSIN(RLAT2/DPR)
        CDLON=DCOS((RLON1-RLON2)/DPR)
        CRD=SLAT1*SLAT2+CLAT1*CLAT2*CDLON
        DISTANCE=SNGL(RERTH*DACOS(CRD)/1000.)
      ENDIF
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      END
