      subroutine gridcheck_nests()
********************************************************************************
*                                                                              *
*     This routine checks the grid specification for the nested model domains. *
*     It is similar to subroutine gridcheck, which checks the mother domain.   *
*                                                                              *
*     Authors: A. Stohl, G. Wotawa                                             *
*                                                                              *
*     8 February 1999                                                          *
********************************************************************************
*  CHANGE: 11/01/2008, Harald Sodemann, GRIB1/2 input with ECMWF grib_api      *
*  CHANGE: 03/12/2008, Harald Sodemann, change to f90 grib_api                 *
********************************************************************************

      use grib_api

      include 'includepar'
      include 'includecom'

!HSO  parameters for grib_api 
      integer ifile
      integer iret
      integer igrib
      character*24 gribErrorMsg
      character*20 gribFunction
!HSO  end
      integer i,j,k,l,ifn,ifield,iumax,iwmax,numskip,nlev_ecn
      integer nuvzn,nwzn
      real akmn(nwzmax),bkmn(nwzmax),akzn(nuvzmax),bkzn(nuvzmax)
      real*4 xaux1,xaux2,yaux1,yaux2
      real*8 xaux1in,xaux2in,yaux1in,yaux2in

C VARIABLES AND ARRAYS NEEDED FOR GRIB DECODING

C dimension of isec2 at least (22+n), where n is the number of parallels or
C meridians in a quasi-regular (reduced) Gaussian or lat/long grid

C dimension of zsec2 at least (10+nn), where nn is the number of vertical
C coordinate parameters

      integer isec1(56),isec2(22+nxmaxn+nymaxn)
      real*4 zsec2(60+2*nuvzmax),zsec4(jpunp)
c     character*1 yoper
c     data yoper/'D'/

!HSO  grib api error messages
      data gribErrorMsg/'Error reading grib file'/
      data gribFunction/'gridcheck_nests'/

      xresoln(0)=1.       ! resolution enhancement for mother grid
      yresoln(0)=1.       ! resolution enhancement for mother grid

C Loop about all nesting levels
*******************************

      do 300 l=1,numbnests

        iumax=0
        iwmax=0

        if(ideltas.gt.0) then
          ifn=1
        else
          ifn=numbwf
        endif
*
* OPENING OF DATA FILE (GRIB CODE)
*
      ifile=0
      igrib=0
      iret=0

5     call grib_open_file(ifile,path(numpath+2*(l-1)+1)
     +  (1:len(numpath+2*(l-1)+1))//trim(wfnamen(l,ifn)),'r',iret)
      if (iret.ne.GRIB_SUCCESS) then
        goto 999   ! ERROR DETECTED
      endif
!     turn on support for multi fields messages
!     call grib_multi_support_on()

      ifield=0   
10    ifield=ifield+1

*
* GET NEXT FIELDS
*
      call grib_new_from_file(ifile,igrib,iret)      
      if (iret.eq.GRIB_END_OF_FILE)  then         
        goto 30    ! EOF DETECTED
      elseif (iret.ne.GRIB_SUCCESS) then
        goto 999   ! ERROR DETECTED
      endif

C Check whether we are on a little endian or on a big endian computer
C *******************************************************************

c         if (inbuff(1).eq.1112101447) then         ! little endian, swap bytes
c           iswap=1+ilen/4
c           call swap32(inbuff,iswap)
c         else if (inbuff(1).ne.1196575042) then    ! big endian
c           stop 'subroutine gridcheck: corrupt GRIB data'
c         endif

          call grib_get_int(igrib,'indicatorOfParameter',isec1(6),iret)
          call grib_check(iret,gribFunction,gribErrorMsg)
          call grib_get_int(igrib,'level',isec1(8),iret)
          call grib_check(iret,gribFunction,gribErrorMsg)

!         get the size and data of the values array
          call grib_get_real4_array(igrib,'values',zsec4,iret)
          call grib_check(iret,gribFunction,gribErrorMsg)

          if(ifield.eq.1) then

!HSO  get the required fields from section 2 in a gribex compatible manner
        call grib_get_int(igrib,'numberOfPointsAlongAParallel',
     >       isec2(2),iret)
        call grib_check(iret,gribFunction,gribErrorMsg)
        call grib_get_int(igrib,'numberOfPointsAlongAMeridian',
     >       isec2(3),iret)
        call grib_check(iret,gribFunction,gribErrorMsg)
        call grib_get_real8(igrib,'longitudeOfFirstGridPointInDegrees',
     >       xaux1in,iret)
        call grib_check(iret,gribFunction,gribErrorMsg)
        call grib_get_real8(igrib,'longitudeOfLastGridPointInDegrees',
     >       xaux2in,iret)
        call grib_check(iret,gribFunction,gribErrorMsg)
        call grib_get_real8(igrib,'latitudeOfLastGridPointInDegrees',
     >       yaux1in,iret)
        call grib_check(iret,gribFunction,gribErrorMsg)
        call grib_get_real8(igrib,'latitudeOfFirstGridPointInDegrees',
     >       yaux2in,iret)
        call grib_check(iret,gribFunction,gribErrorMsg)
        call grib_get_int(igrib,'numberOfVerticalCoordinateValues',
     >       isec2(12),iret)
        call grib_check(iret,gribFunction,gribErrorMsg)

            nxn(l)=isec2(2)
            nyn(l)=isec2(3)
            xaux1=xaux1in
            xaux2=xaux2in
            yaux1=yaux1in
            yaux2=yaux2in

!HSO        get the size and data of the vertical coordinate array
            call grib_get_real4_array(igrib,'pv',zsec2,iret)
            call grib_check(iret,gribFunction,gribErrorMsg)

!           xaux1=float(isec2(5))/1000.
!           xaux2=float(isec2(8))/1000.
            if(xaux1.gt.180) xaux1=xaux1-360.0
            if(xaux2.gt.180) xaux2=xaux2-360.0
            if(xaux1.lt.-180) xaux1=xaux1+360.0
            if(xaux2.lt.-180) xaux2=xaux2+360.0
            if (xaux2.lt.xaux1) xaux2=xaux2+360.
!           yaux1=float(isec2(7))/1000.
!           yaux2=float(isec2(4))/1000.
            xlon0n(l)=xaux1
            ylat0n(l)=yaux1
            dxn(l)=(xaux2-xaux1)/float(nxn(l)-1)
            dyn(l)=(yaux2-yaux1)/float(nyn(l)-1)
            nlev_ecn=isec2(12)/2-1
          endif ! ifield.eq.1
     
          k=isec1(8)
          if(isec1(6).eq.131) iumax=max(iumax,nlev_ec-k+1)
          if(isec1(6).eq.135) iwmax=max(iwmax,nlev_ec-k+1) 

          if(isec1(6).eq.129) then
          do 20 j=0,nyn(l)-1
            do 20 i=0,nxn(l)-1
20              oron(i,j,l)=zsec4(nxn(l)*(nyn(l)-j-1)+i+1)/ga
          endif
          if(isec1(6).eq.172) then
          do 25 j=0,nyn(l)-1
            do 25 i=0,nxn(l)-1
25              lsmn(i,j,l)=zsec4(nxn(l)*(nyn(l)-j-1)+i+1)/ga
          endif
          if(isec1(6).eq.160) then
          do 26 j=0,nyn(l)-1
            do 26 i=0,nxn(l)-1
26              excessoron(i,j,l)=zsec4(nxn(l)*(nyn(l)-j-1)+i+1)/ga
          endif

          call grib_release(igrib)
          goto 10                 !! READ NEXT LEVEL OR PARAMETER
*
* CLOSING OF INPUT DATA FILE
*

30    call grib_close_file(ifile)

        nuvzn=iumax
        nwzn=iwmax
        if(nuvzn.eq.nlev_ec) nwzn=nlev_ecn+1

        if (nxn(l).gt.nxmaxn) then
      write(*,*) 'FLEXPART error: Too many grid points in x direction.'
      write(*,*) 'Reduce resolution of wind fields (file GRIDSPEC)'
      write(*,*) 'for nesting level ',l
      write(*,*) 'Or change parameter settings in file includepar.'
      write(*,*) nxn(l),nxmaxn
          stop
        endif

        if (nyn(l).gt.nymaxn) then
      write(*,*) 'FLEXPART error: Too many grid points in y direction.'
      write(*,*) 'Reduce resolution of wind fields (file GRIDSPEC)'
      write(*,*) 'for nesting level ',l
      write(*,*) 'Or change parameter settings in file includepar.'
      write(*,*) nyn(l),nymaxn
          stop
        endif

        if ((nuvzn.gt.nuvzmax).or.(nwzn.gt.nwzmax)) then
      write(*,*) 'FLEXPART error: Nested wind fields have too many'//
     +'vertical levels.'
      write(*,*) 'Problem was encountered for nesting level ',l
          stop
        endif


C Output of grid info
*********************

      write(*,'(a,i2)') 'Nested domain #: ',l
      write(*,'(a,f10.1,a1,f10.1,a,f10.1)') '  Longitude range: ',
     +xlon0n(l),' to ',xlon0n(l)+(nxn(l)-1)*dxn(l),
     +'   Grid distance: ',dxn(l)
      write(*,'(a,f10.1,a1,f10.1,a,f10.1)') '  Latitude range: ',
     +ylat0n(l),' to ',ylat0n(l)+(nyn(l)-1)*dyn(l),
     +'   Grid distance: ',dyn(l)
      write(*,*)

C Determine, how much the resolutions in the nests are enhanced as
C compared to the mother grid
******************************************************************

        xresoln(l)=dx/dxn(l)
        yresoln(l)=dy/dyn(l)

C Determine the mother grid coordinates of the corner points of the
C nested grids
C Convert first to geographical coordinates, then to grid coordinates
*********************************************************************

        xaux1=xlon0n(l)
        xaux2=xlon0n(l)+float(nxn(l)-1)*dxn(l)
        yaux1=ylat0n(l)
        yaux2=ylat0n(l)+float(nyn(l)-1)*dyn(l)

        xln(l)=(xaux1-xlon0)/dx
        xrn(l)=(xaux2-xlon0)/dx
        yln(l)=(yaux1-ylat0)/dy
        yrn(l)=(yaux2-ylat0)/dy


        if ((xln(l).lt.0.).or.(yln(l).lt.0.).or.
     +  (xrn(l).gt.float(nxmin1)).or.(yrn(l).gt.float(nymin1))) then
          write(*,*) 'Nested domain does not fit into mother domain'
          write(*,*) 'For global mother domain fields, you can shift'
          write(*,*) 'shift the mother domain into x-direction'
          write(*,*) 'by setting nxshift (file includepar) to a'
          write(*,*) 'positive value. Execution is terminated.'
          stop
        endif


* CALCULATE VERTICAL DISCRETIZATION OF ECMWF MODEL
* PARAMETER akm,bkm DESCRIBE THE HYBRID "ETA" COORDINATE SYSTEM

        numskip=nlev_ecn-nuvzn ! number of ecmwf model layers not used by FLEXPART
        do 40 i=1,nwzn
          j=numskip+i
          k=nlev_ecn+1+numskip+i
          akmn(nwzn-i+1)=zsec2(j)
40        bkmn(nwzn-i+1)=zsec2(k)

*
* CALCULATION OF AKZ, BKZ
* AKZ,BKZ: model discretization parameters at the center of each model
*          layer
*
C Assign the 10 m winds to an artificial model level with akz=0 and bkz=1.0,
C i.e. ground level
******************************************************************************
 
        akzn(1)=0.
        bkzn(1)=1.0
        do 90 i=1,nuvzn
          akzn(i+1)=0.5*(akmn(i+1)+akmn(i))
          bkzn(i+1)=0.5*(bkmn(i+1)+bkmn(i))
90        continue
        nuvzn=nuvzn+1

C Check, whether the heights of the model levels of the nested
C wind fields are consistent with those of the mother domain.
C If not, terminate model run.
**************************************************************

        do 75 i=1,nuvz
          if ((akzn(i).ne.akz(i)).or.(bkzn(i).ne.bkz(i))) then
      write(*,*) 'FLEXPART error: The wind fields of nesting level',l
      write(*,*) 'are not consistent with the mother domain:'
      write(*,*) 'Differences in vertical levels detected.'
            stop
          endif
75        continue

        do 85 i=1,nwz
          if ((akmn(i).ne.akm(i)).or.(bkmn(i).ne.bkm(i))) then
      write(*,*) 'FLEXPART error: The wind fields of nesting level',l
      write(*,*) 'are not consistent with the mother domain:'
      write(*,*) 'Differences in vertical levels detected.'
            stop
          endif
85        continue

300     continue

      return

999   write(*,*)  
      write(*,*) ' ###########################################'//
     &           '###### '
      write(*,*) '                FLEXPART SUBROUTINE GRIDCHECK:'
      write(*,*) ' CAN NOT OPEN INPUT DATA FILE '//wfnamen(l,ifn)
      write(*,*) ' FOR NESTING LEVEL ',k
      write(*,*) ' ###########################################'//
     &           '###### '
      stop

      end
